@@ -151,7 +151,7 @@ class IndexedArray(awkward.array.base.AwkwardArrayWithContent):
 
         if self._util_isstringslice(where):
             content = self._content[where]
-            cls = awkward.array.objects.Methods.maybemixin(type(content), IndexedArray)
+            cls = awkward.array.objects.Methods.maybemixin(type(content), self.IndexedArray)
             out = cls.__new__(cls)
             out.__dict__.update(self.__dict__)
             out._content = content
@@ -411,7 +411,7 @@ class SparseArray(awkward.array.base.AwkwardArrayWithContent):
 
         if self._util_isstringslice(where):
             content = self._content[where]
-            cls = awkward.array.objects.Methods.maybemixin(type(content), SparseArray)
+            cls = awkward.array.objects.Methods.maybemixin(type(content), self.SparseArray)
             out = cls.__new__(cls)
             out.__dict__.update(self.__dict__)
             out._content = content
