@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2018, DIANA-HEP
+# Copyright (c) 2019, IRIS-HEP
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -42,12 +42,13 @@ try:
 except ImportError:
     from collections import Mapping, MutableMapping
 
+import numpy
+
 import awkward.type
-import awkward.util
 import awkward.version
 
 compression = [
-    {"minsize": 8192, "types": [awkward.util.numpy.bool_, awkward.util.numpy.bool, awkward.util.numpy.integer], "contexts": "*", "pair": (zlib.compress, ("zlib", "decompress"))},
+    {"minsize": 8192, "types": [numpy.bool_, numpy.bool, numpy.integer], "contexts": "*", "pair": (zlib.compress, ("zlib", "decompress"))},
     ]
 
 partner = {
@@ -59,6 +60,7 @@ whitelist = [["awkward.util", "frombuffer"],
              ["zlib", "decompress"],
              ["awkward", "*Array"],
              ["awkward", "Table"],
+             ["awkward", "numpy", "frombuffer"],
              ["awkward.persist", "*"],
              ["awkward.arrow", "ParquetFile", "fromjson"]]
 
@@ -68,12 +70,14 @@ def frompython(obj):
 def topython(string):
     return pickle.loads(base64.b64decode(string.encode("ascii")))
 
-def spec2function(obj, whitelist=whitelist):
+def spec2function(obj, awkwardlib="awkward", whitelist=whitelist):
     for white in whitelist:
         for n, p in zip(obj, white):
             if not fnmatch.fnmatchcase(n, p):
                 break
         else:
+            if obj[0] == "awkward":
+                obj = [awkwardlib] + obj[1:]
             gen, genname = importlib.import_module(obj[0]), obj[1:]
             while len(genname) > 0:
                 gen, genname = getattr(gen, genname[0]), genname[1:]
@@ -99,7 +103,7 @@ def json2dtype(obj):
             return [recurse(x) for x in obj]
         else:
             return obj
-    return awkward.util.numpy.dtype(recurse(obj))
+    return numpy.dtype(recurse(obj))
 
 def type2json(obj):
     if isinstance(obj, awkward.type.Type):
@@ -142,7 +146,7 @@ def type2json(obj):
                 else:
                     return out
 
-        elif isinstance(obj, awkward.util.numpy.dtype):
+        elif isinstance(obj, numpy.dtype):
             return {"dtype": dtype2json(obj)}
 
         elif callable(obj):
@@ -226,13 +230,13 @@ def jsonable(obj):
     elif isinstance(obj, str):
         return str(obj)
 
-    elif isinstance(obj, (bool, awkward.util.numpy.bool_, awkward.util.numpy.bool)):
+    elif isinstance(obj, (bool, numpy.bool_, numpy.bool)):
         return bool(obj)      # policy: eliminate Numpy types
 
-    elif isinstance(obj, (numbers.Integral, awkward.util.numpy.integer)):
+    elif isinstance(obj, (numbers.Integral, numpy.integer)):
         return int(obj)       # policy: eliminate Numpy types
 
-    elif isinstance(obj, (numbers.Real, awkward.util.numpy.floating)) and awkward.util.numpy.isfinite(obj):
+    elif isinstance(obj, (numbers.Real, numpy.floating)) and numpy.isfinite(obj):
         return float(obj)     # policy: eliminate Numpy types
 
     else:
@@ -298,12 +302,12 @@ def serialize(obj, storage, name=None, delimiter="-", suffix=None, schemasuffix=
         ident = len(seen)
         seen[id(obj)] = ident
 
-        if type(obj) is awkward.util.numpy.dtype:
+        if type(obj) is numpy.dtype:
             return {"dtype": dtype2json(obj)}
 
-        elif type(obj) is awkward.util.numpy.ndarray and len(obj.shape) != 0:
+        elif type(obj) is numpy.ndarray and len(obj.shape) != 0:
             if len(obj.shape) > 1:
-                dtype = awkward.util.numpy.dtype((obj.dtype, obj.shape[1:]))
+                dtype = numpy.dtype((obj.dtype, obj.shape[1:]))
             else:
                 dtype = obj.dtype
 
@@ -314,7 +318,7 @@ def serialize(obj, storage, name=None, delimiter="-", suffix=None, schemasuffix=
                     storage[prefix + str(ident) + suffix] = compress(obj)
 
                     return {"id": ident,
-                            "call": ["awkward.util", "frombuffer"],
+                            "call": ["awkward", "numpy", "frombuffer"],
                             "args": [{"call": decompress, "args": [{"read": str(ident) + suffix}]},
                                      {"dtype": dtype2json(dtype)},
                                      {"json": len(obj)}]}
@@ -322,7 +326,7 @@ def serialize(obj, storage, name=None, delimiter="-", suffix=None, schemasuffix=
             else:
                 storage[prefix + str(ident) + suffix] = obj.tostring()
                 return {"id": ident,
-                        "call": ["awkward.util", "frombuffer"],
+                        "call": ["awkward", "numpy", "frombuffer"],
                         "args": [{"read": str(ident) + suffix},
                                  {"dtype": dtype2json(dtype)},
                                  {"json": len(obj)}]}
@@ -369,11 +373,11 @@ def serialize(obj, storage, name=None, delimiter="-", suffix=None, schemasuffix=
     storage[name + schemasuffix] = json.dumps(schema).encode("ascii")
     return schema
 
-def deserialize(storage, name="", whitelist=whitelist, cache=None):
+def deserialize(storage, name="", awkwardlib="awkward", whitelist=whitelist, cache=None):
     import awkward.array.virtual
 
     schema = storage[name]
-    if isinstance(schema, awkward.util.numpy.ndarray):
+    if isinstance(schema, numpy.ndarray):
         schema = schema.tostring()
     if isinstance(schema, bytes):
         schema = schema.decode("ascii")
@@ -393,7 +397,7 @@ def deserialize(storage, name="", whitelist=whitelist, cache=None):
     def unfill(schema):
         if isinstance(schema, dict):
             if "call" in schema and isinstance(schema["call"], list) and len(schema["call"]) > 0:
-                gen = spec2function(schema["call"], whitelist=whitelist)
+                gen = spec2function(schema["call"], awkwardlib=awkwardlib, whitelist=whitelist)
                 args = [unfill(x) for x in schema.get("args", [])]
 
                 kwargs = {}
@@ -428,7 +432,7 @@ def deserialize(storage, name="", whitelist=whitelist, cache=None):
                 out = json2dtype(schema["dtype"])
 
             elif "function" in schema:
-                out = spec2function(schema["function"], whitelist=whitelist)
+                out = spec2function(schema["function"], awkwardlib=awkwardlib, whitelist=whitelist)
 
             elif "json" in schema:
                 out = schema["json"]
@@ -459,7 +463,7 @@ def deserialize(storage, name="", whitelist=whitelist, cache=None):
 
 def keys(storage, name="", subschemas=True):
     schema = storage[name]
-    if isinstance(schema, awkward.util.numpy.ndarray):
+    if isinstance(schema, numpy.ndarray):
         schema = schema.tostring()
     if isinstance(schema, bytes):
         schema = schema.decode("ascii")
@@ -594,13 +598,13 @@ class Load(Mapping):
 
         self._file = Wrap()
 
-        alloptions = {"schemasuffix": ".json", "whitelist": whitelist, "cache": None}
+        alloptions = {"schemasuffix": ".json", "awkwardlib": "awkward", "whitelist": whitelist, "cache": None}
         alloptions.update(options)
         self.schemasuffix = alloptions.pop("schemasuffix")
         self.options = alloptions
 
     def __getitem__(self, where):
-        return deserialize(self._file, name=where + self.schemasuffix, whitelist=self.options["whitelist"], cache=self.options["cache"])
+        return deserialize(self._file, name=where + self.schemasuffix, awkwardlib=self.options["awkwardlib"], whitelist=self.options["whitelist"], cache=self.options["cache"])
 
     def __iter__(self):
         for n in self._file.f.namelist():
@@ -631,7 +635,7 @@ class Load(Mapping):
 
 class hdf5(MutableMapping):
     def __init__(self, group, **options):
-        alloptions = {"compression": compression, "whitelist": whitelist, "cache": None}
+        alloptions = {"compression": compression, "awkwardlib": "awkward", "whitelist": whitelist, "cache": None}
         alloptions.update(options)
         self.options = alloptions
         self.options["delimiter"] = "/"
@@ -641,17 +645,19 @@ class hdf5(MutableMapping):
             def __init__(self):
                 self.g = group
             def __getitem__(self, where):
-                return self.g[where].value
+                return self.g[where][()]
             def __setitem__(self, where, what):
-                self.g[where] = awkward.util.numpy.frombuffer(what, dtype=awkward.util.numpy.uint8)
+                self.g[where] = numpy.frombuffer(what, dtype=numpy.uint8)
 
         self._group = Wrap()
 
     def __getitem__(self, where):
-        return deserialize(self._group, name=where + self.options["schemasuffix"], whitelist=self.options["whitelist"], cache=self.options["cache"])
+        return deserialize(self._group, name=where + self.options["schemasuffix"], awkwardlib=self.options["awkwardlib"], whitelist=self.options["whitelist"], cache=self.options["cache"])
 
     def __setitem__(self, where, what):
         options = dict(self.options)
+        if "awkwardlib" in options:
+            del options["awkwardlib"]
         if "whitelist" in options:
             del options["whitelist"]
         if "cache" in options:
